Package: RTCGAToolbox
Type: Package
Title: A new tool for exporting TCGA Firehose data
Version: 2.40.0
Authors@R: c(
    person("Mehmet", "Samur", email = "rtcgatoolbox@gmail.com",
    role = "aut"),
    person("Marcel", "Ramos", email = "marcel.ramos@sph.cuny.edu",
    role = c("aut", "cre"), comment = c(ORCID = "0000-0002-3242-0582")),
    person("Ludwig", "Geistlinger", role = "ctb"))
Description: Managing data from large scale projects such as The Cancer Genome
    Atlas (TCGA) for further analysis is an important and time consuming step
    for research projects. Several efforts, such as Firehose project, make TCGA
    pre-processed data publicly available via web services and data portals but
    it requires managing, downloading and preparing the data for following
    steps. We developed an open source and extensible R based data client for
    Firehose pre-processed data and demonstrated its use with sample case
    studies. Results showed that RTCGAToolbox could improve data management for
    researchers who are interested with TCGA data. In addition, it can be
    integrated with other analysis pipelines for following data analysis.
License: GPL-2
Depends: R (>= 4.3.0)
Imports: BiocGenerics, data.table, DelayedArray, GenomicRanges,
        Seqinfo, httr, methods, RaggedExperiment, RCurl, RJSONIO,
        rvest, S4Vectors, stats, stringr, SummarizedExperiment,
        TCGAutils, utils
Suggests: BiocStyle, Homo.sapiens, knitr, readr, rmarkdown
biocViews: DifferentialExpression, GeneExpression, Sequencing
URL: http://mksamur.github.io/RTCGAToolbox/
BugReports: https://github.com/mksamur/RTCGAToolbox/issues
VignetteBuilder: knitr
RoxygenNote: 7.3.2
Roxygen: list(markdown = TRUE)
Collate: 'RTCGAToolbox-Class.R' 'RTCGAToolbox.R' 'utils.R'
        'biocExtract.R' 'data.R' 'getBroadSubtypes.R'
        'getFirehoseAnalyzeDates.R' 'getFirehoseData.R'
        'getFirehoseDatasets.R' 'getFirehoseRunningDates.R'
        'getGISTICPeaks.R' 'getLinks.R' 'getMutationRate.R'
        'makeSummarizedExperimentFromGISTIC.R' 'selectType.R'
Encoding: UTF-8
Date: 2024-12-18
git_url: https://git.bioconductor.org/packages/RTCGAToolbox
git_branch: RELEASE_3_22
git_last_commit: 7f9df8a
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 06:10:49 UTC; biocbuild
Author: Mehmet Samur [aut],
  Marcel Ramos [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3242-0582>),
  Ludwig Geistlinger [ctb]
Maintainer: Marcel Ramos <marcel.ramos@sph.cuny.edu>
