# Generated by roxygen2: do not edit by hand

export(biocExtract)
export(getBroadSubtypes)
export(getFirehoseAnalyzeDates)
export(getFirehoseData)
export(getFirehoseDatasets)
export(getFirehoseRunningDates)
export(getGISTICPeaks)
export(getLinks)
export(getMutationRate)
export(makeSummarizedExperimentFromGISTIC)
exportClasses(CorResult)
exportClasses(DGEResult)
exportClasses(FirehoseCGHArray)
exportClasses(FirehoseData)
exportClasses(FirehoseGISTIC)
exportClasses(FirehoseMethylationArray)
exportClasses(FirehosemRNAArray)
exportMethods(getData)
exportMethods(isEmpty)
exportMethods(selectType)
exportMethods(showResults)
exportMethods(updateObject)
importFrom(BiocGenerics,updateObject)
importFrom(DelayedArray,DelayedArray)
importFrom(Seqinfo,"seqlevels<-")
importFrom(Seqinfo,orderSeqlevels)
importFrom(Seqinfo,seqlevels)
importFrom(GenomicRanges,makeGRangesFromDataFrame)
importFrom(GenomicRanges,makeGRangesListFromDataFrame)
importFrom(GenomicRanges,order)
importFrom(GenomicRanges,seqnames)
importFrom(RCurl,getURL)
importFrom(RCurl,url.exists)
importFrom(RJSONIO,fromJSON)
importFrom(RaggedExperiment,RaggedExperiment)
importFrom(S4Vectors,"mcols<-")
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,SimpleList)
importFrom(S4Vectors,isEmpty)
importFrom(S4Vectors,isSingleString)
importFrom(S4Vectors,mcols)
importFrom(S4Vectors,metadata)
importFrom(S4Vectors,splitAsList)
importFrom(SummarizedExperiment,"rowData<-")
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,makeSummarizedExperimentFromDataFrame)
importFrom(data.table,fread)
importFrom(methods,.hasSlot)
importFrom(methods,as)
importFrom(methods,callNextMethod)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,slotNames)
importFrom(stats,na.omit)
importFrom(stringr,str_extract)
importFrom(utils,download.file)
importFrom(utils,head)
importFrom(utils,read.delim)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
importFrom(utils,type.convert)
importFrom(utils,untar)
importFrom(utils,write.table)
