#' A subset of the Adrenocortical Carcinoma (ACC) dataset
#'
#' See the `acc_sample.R` script to see how the data was generated.
#' This dataset contains real data from the The Cancer Genome Atlas for
#' the pipeline run date and GISTIC analysis date of 2016-01-28.
#'
#' @format A `FirehoseData` data object
#'
#' @usage data("accmini", package = "RTCGAToolbox")
#'
#' @docType data
#'
"accmini"
